/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/pgmspace.h>
#include <stdio.h>
#include "typedef.h"
#include "sysdef.h"
#include "drvlcd.h"
#include "drvtimer.h"
#include "drveeprom.h"
#include "drvtemp.h"
#include "display.h"
#include "drvheat.h"

#define TIMER_STATE_OFF     0
#define TIMER_STATE_RUN     1
#define TIMER_STATE_ELAPSED 2


static E_DISPLAY_MODES uDisplayMode;
static u08 uTimerState = TIMER_STATE_OFF;
static T_EEPROM *sEeprom;
static T_PARAM_ALL *sParamAll;
static u16 uOldTist[E_SGUN_MAX];
static u16 uOldTsoll[E_SGUN_MAX];
static u08 uOldPower[E_SGUN_MAX] = {200,200};
static u08 uOldActGun = 0xff;

static void DisplayTimerElapsed( void );
static void ModeStartup( void );
static void ModeNom1( void );

void display_InitModule( void )
{
  u08 n;
  
  sParamAll = drveeprom_GetParameter();
  sEeprom = sParamAll->tEeprom;
  for(n=0; n<E_SGUN_MAX; n++)
  {
    uOldTist[n] = 0;
    uOldTsoll[n] = 0;
    uOldPower[n] = -1;
  }
  uDisplayMode = E_DISPLAY_MODE_STARTUP;
}

void display_Task( void )
{
  
  switch( uDisplayMode )
  {
  case E_DISPLAY_MODE_STARTUP:
    ModeStartup();
    break;
  case E_DISPLAY_MODE_NOM_1:
    ModeNom1();
    break;
  case E_DISPLAY_MODE_NOM_2:
    //ModeNom2();
    break;
  case E_DISPLAY_MODE_ERROR:
    //do nothing wait for a key
    break;
  default:
    uDisplayMode = E_DISPLAY_MODE_NOM_1;
  }
}

void display_SetErrorText( char *t1, char *t2)
{
  lcdClear();
  lcd_putstr((u08*)t1);
  lcdGotoXY(1,2);
  lcd_putstr((u08*)t2);
  uDisplayMode = E_DISPLAY_MODE_ERROR;
}

void display_ResetErrorText( void )
{
  uDisplayMode = E_DISPLAY_MODE_NOM_1;
}

u08 display_ErrorActive( void )
{
  return (uDisplayMode == E_DISPLAY_MODE_ERROR);
}

static void ModeStartup( void )
{
  switch( uTimerState )
  {
  case TIMER_STATE_OFF:
    lcdClear();
    lcd_putstr_P(PSTR("Loetstation V1.2"));
    lcdGotoXY(1,2);
    lcd_putstr_P(PSTR("AVR Inside      "));
    uTimerState = TIMER_STATE_RUN;
    drvtim_TimerAdd( DisplayTimerElapsed, 5000 );
    break;
    
  case TIMER_STATE_RUN:
    break;

  case TIMER_STATE_ELAPSED:
    uTimerState = TIMER_STATE_OFF;
    if( drveeprom_ChecksumError() )
    {
      display_SetErrorText("Err: EEPROM data","Press any key...");
    }
    else
      uDisplayMode = E_DISPLAY_MODE_NOM_1;
    break;
  }  
}

static void ModeNom1( void )
{
/*
 * Nominal Screen 1       Change Temp.
 * +----------------+     +----------------+
 * |>I:320   I:380|     |>S:330   I:380|
 * | P: 15%   P: 22%|     | P: 15%   P: 22%|
 * +----------------+     +----------------+
 */
  static u08 uOffDetected[E_SGUN_MAX] = {0,0};
  u08 uGunIsOff[E_SGUN_MAX];
  u16 tist[E_SGUN_MAX];
  u16 tsoll[E_SGUN_MAX];
  u08 uPower[E_SGUN_MAX];
  u08 n = 0;
  u16 temp;
  char s[20];
  char cActGun1,cActGun2;
  u08 uChanged = FALSE;
  u08 uSollChanged = FALSE;
  u08 uPowerChanged[E_SGUN_MAX] = {FALSE,FALSE};
  
  for(n=0; n<sEeprom->uUsedGuns;n++)
  {
    //--- read temperature  
    tist[n] = drvtemp_ReadTemp( n, &temp );
    if( uOldTist[n] != tist[n] )
      uChanged = TRUE;
    uOldTist[n] = tist[n];
    
    //--- read nominal value
    tsoll[n] = drveeprom_GetParameter()->tEeprom->uNominalTemp[n];
    if( uOldTsoll[n] != tsoll[n] )
      uSollChanged = TRUE;
    uOldTsoll[n] = tsoll[n];
     
    //--- read actual heating power
    uPower[n] = drvheat_GetHeatingPower(n);
    if( uOldPower[n] != uPower[n] )
      uPowerChanged[n] = TRUE;
    uOldPower[n] = uPower[n];
    
  }
  
  if( uOldActGun != sParamAll->uActGun )
  {
    uSollChanged = TRUE; // show also target values if gun was changed
  }
  uOldActGun = sParamAll->uActGun;
  
  cActGun1 = sParamAll->uActGun == 0 ? '>' : ' ';
  cActGun2 = sParamAll->uActGun == 1 ? '>' : ' ';

  if( uTimerState == TIMER_STATE_ELAPSED )
  {
    uTimerState = TIMER_STATE_OFF;
    uChanged = TRUE;
  }
  
  lcdGotoXY(1,1);
  if( uSollChanged )
  {
    sprintf_P(s,PSTR("%cS:%3d%c  %cS:%3d%c"),cActGun1,tsoll[0],CHR_CELSIUS,cActGun2,tsoll[1],CHR_CELSIUS);
    lcd_putstr((u08*)s);
    uTimerState = TIMER_STATE_RUN;
    drvtim_TimerRemove( DisplayTimerElapsed );
    drvtim_TimerAdd( DisplayTimerElapsed, 2000 );
  }
  else if( uChanged && uTimerState == TIMER_STATE_OFF )
  {
    sprintf_P(s,PSTR("%cI:%3d%c  %cI:%3d%c"),cActGun1,tist[0],CHR_CELSIUS,cActGun2,tist[1],CHR_CELSIUS);
    lcd_putstr((u08*)s);
  }
  
  for(n=0; n<sEeprom->uUsedGuns;n++)
  {
    uGunIsOff[n] = drveeprom_GetParameter()->tEeprom->uGunOff[n];
    
    if( uGunIsOff[n] && !uOffDetected[n] )
    {
      lcdGotoXY(n == E_SGUN1 ? 1 : 10, 2);
      sprintf_P(s,PSTR(" P:OFF    "));
      lcd_putstr((u08*)s);
    }
    else if ( uPowerChanged[n] && !uGunIsOff[n] )
    {
      lcdGotoXY(n == E_SGUN1 ? 1 : 10, 2);
      sprintf_P(s,PSTR(" P:%3d%%   "),uPower[n]);
      lcd_putstr((u08*)s);
    }
    uOffDetected[n] = uGunIsOff[n];
  }
}

static void DisplayTimerElapsed( void )
{
  uTimerState = TIMER_STATE_ELAPSED;
}


