/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <avr/io.h>
#include <avr/eeprom.h>
#include "typedef.h"
#include "sysdef.h"
#include "drveeprom.h"
#include "drvtimer.h"

static T_EEPROM sEepromData;
static T_PARAM_ALL sParamAll;

static u16 ee_ptr;
static u08 uEepromCheckumError = FALSE;

//#define EE_START ((u16)&sEepromData)
//#define EE_END   ((u16)(&sEepromData + sizeof(T_EEPROM)))
#define EE_START 0
#define EE_END   (sizeof(T_EEPROM))

// function prototypes
static u16 CalcCrc16( void );
static void InitEeprom( void );
unsigned int calcCRC16r(unsigned int crc, unsigned int c);
static void StartEepromWrite( void );

// return: 0 =OK
//         1 = EEPROM content was defect
u08 drveeprom_InitModule( void )
{
  u08 *ptr = (u08*)&sEepromData;
  u16 ptr_ee = EE_START;
  u08 ret;
  
  sParamAll.tEeprom = &sEepromData;
  sParamAll.uActGun = 0;
  
  do
  {
    EEARL = ((u16)ptr_ee) & 0xff;
    EEARH = (((u16)ptr_ee) >> 8) & 0xff;
    EECR |= 1<<EERE;  // read
    *ptr = EEDR;
    ptr_ee++;
    ptr++;
  }
  while( ptr_ee < sizeof(sEepromData) );
  ee_ptr = EE_END+1;  // init
  
  if( CalcCrc16() != sEepromData.uCrc )
  {
    uEepromCheckumError = TRUE;
    ret = 1;
    InitEeprom();
  }
  else
    ret = 0;
    
  return ret;
}

T_PARAM_ALL *drveeprom_GetParameter( void )
{
  return &sParamAll;
}

u08 drveeprom_ChecksumError( void )
{
  return uEepromCheckumError;
}

// write back needed
void drveeprom_Write( void )
{
  sEepromData.uCrc = CalcCrc16();  // recalculate crc
  drvtim_TimerRemove(StartEepromWrite);
  drvtim_TimerAdd( StartEepromWrite, 5000 );
}

// start write back now
static void StartEepromWrite( void )
{
  ee_ptr = EE_START;
}

// rewrite changed data
void drveeprom_Update( void )
{
  u08 val;
  
//  if( EECR & 1<<EEWE )                  // previous write not finished
//    return;
  EEARL = 0;                            // address outside used data

  if( ee_ptr > EE_END )                 // update done
    return;

  val = ((u08*)&sEepromData)[ee_ptr-EE_START];       // update
  if( eeprom_read_byte((u08*)ee_ptr) != val )
  {  // if changed
    eeprom_write_byte((u08*)ee_ptr,val);
  }
  ee_ptr++;                                     // next address
}

static void InitEeprom( void )
{
//#define DEF_KP 2.5
//#define DEF_KI 0.2
//#define DEF_KD 2.0
#define DEF_KP 100
#define DEF_KI 18
#define DEF_KD 25
  
  sEepromData.uUsedGuns  = 2;
  sEepromData.uGunOff[0] = TRUE;
  sEepromData.uGunOff[1] = TRUE;
  sEepromData.uNomScreen = 1;
  sEepromData.uCalLow[0] = 0;
  sEepromData.uCalLow[1] = 0;
  sEepromData.uCalHigh[0] = 0;
  sEepromData.uCalHigh[1] = 0;
  sEepromData.uNominalTemp[0]  = 180;
  sEepromData.uNominalTemp[1]  = 180;
  sEepromData.fKp[0]     = DEF_KP; 
  sEepromData.fKp[1]     = DEF_KP; 
  sEepromData.fKi[0]     = DEF_KI; 
  sEepromData.fKi[1]     = DEF_KI; 
  sEepromData.fKd[0]     = DEF_KD; 
  sEepromData.fKd[1]     = DEF_KD;
  sEepromData.uCrc       = 0;
  drveeprom_Write();
}


static u16 CalcCrc16( void )
{
  u16 uCrc = 0;
  u08 n;
  
  for(n=0; n < (sizeof(T_EEPROM)-sizeof(u16)); n++)
    uCrc = calcCRC16r( uCrc, (u16)(((u08*)&sEepromData)[n]));
  return uCrc;
}

unsigned int calcCRC16r(unsigned int crc, unsigned int c)
{ // CCITT 16 bit (X^16 + X^12 + X^5 + 1).
  crc  = (unsigned char)(crc >> 8) | (crc << 8);
  crc ^= c;
  crc ^= (unsigned char)(crc & 0xff) >> 4;
  crc ^= (crc << 8) << 4;
  crc ^= ((crc & 0xff) << 4) << 1;

  return (crc);
}


