/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include "typedef.h"
#include "sysdef.h"
#include "hwdef.h"
#include "drvkeys.h"

#define BTN_SHORT  5   // 50ms
#define BTN_LONG   75   // 750ms
#define BTN_LONG_MASK     0x01  // bit may not conflict with bits HW push buttons
#define BTN_REPEAT_MASK   0x02  // bit may not conflict with bits HW push buttons
#define BTN_PUSH_ENC_BIT  0x08  // bit may not conflict with bits HW push buttons

static volatile s08 enc_delta;    // -128 ... 127
static volatile u08 uButtons = 0;

static void ReadEncoder( void );
static void ReadPushButtons( void );
static int GetEncoder( void );

void drvkeys_InitModule( void )
{
//--- rotary encoder port init
  // enable pullup
  ROT_ENC_PU_A     = 1;  // signal A
  ROT_ENC_PU_B     = 1;  // signal B
  ROT_ENC_PU_PUSH  = 1;  // push button
  // set as input
  ROT_ENC_DDR_SIGA = 0;  
  ROT_ENC_DDR_SIGB = 0;  
  ROT_ENC_DDR_PUSH = 0;

//--- push buttons port init
  // enable pullups
  GEN_PUSH_PU_BT1 = 1;
  GEN_PUSH_PU_BT2 = 1;
  GEN_PUSH_PU_BT3 = 1;
  GEN_PUSH_PU_BT4 = 1;
  // set as input
  GEN_PUSH_DDR_BT1 = 0;
  GEN_PUSH_DDR_BT2 = 0;
  GEN_PUSH_DDR_BT3 = 0;
  GEN_PUSH_DDR_BT4 = 0;
  
}

static int GetEncoder( void )
{
  s08 val=0;
  if( abs(enc_delta) >= 4 )
  {
    cli();
    val = enc_delta/4;
    enc_delta = 0; 
    sei();
  }
  return val; 
}

u08 drvkeys_kbhit( void )
{
  u08 ret_val =FALSE;
  do
  {
    if( abs(enc_delta) >= 4 )
    {
      ret_val = TRUE;  // rotary encoder give pulses
      break;
    }  
  
    if( uButtons != 0 )
    {
      ret_val = TRUE;  // a key was pressed
      break;
    }  

  } while(0);
  return ret_val;
}

char drvkeys_getchar( void )
{
  s08 sVal;
  char uKey = 0;
  u08 uButtonsLoc;
  
  while( !drvkeys_kbhit() );
  
  cli();
  uButtonsLoc = uButtons;  // get push buttons
  uButtons = 0;           // clear push buttons
  sei();
  
  do
  {
//--- Check rotary encoder
    sVal = GetEncoder();
    if( sVal < 0 )
    {
      uKey = BUTTON_LEFT;
      break;
    }
    if( sVal > 0 )
    {
      uKey = BUTTON_RIGHT;
      break;
    }
    
//--- Check push buttons
    if( uButtonsLoc )
    {
      switch( uButtonsLoc & (GEN_PUSH_BTN_MASK | BTN_PUSH_ENC_BIT) )
      {
      case GEN_PUSH_BT1_BIT:
        uKey = BUTTON_1;
        break;
      case GEN_PUSH_BT2_BIT:
        uKey = BUTTON_2;
        break;
      case GEN_PUSH_BT3_BIT:
        uKey = BUTTON_3;
        break;
      case GEN_PUSH_BT4_BIT:
        uKey = BUTTON_4;
        break;
      case BTN_PUSH_ENC_BIT:
        uKey = BUTTON_ENTER;
        break;
      }
      if( uKey != 0  && (uButtonsLoc & BTN_LONG_MASK))
      {
        uKey -= 0x20;  // convert to upper case
      }
      break;
    }
  }
  while(0);
  return uKey;  
}

// call with 10kHz (for the encoder it has to be fast)
void drvkeys_task( void )
{
  static u08 ticks = 100;
  ReadEncoder();
  if( !(--ticks) )
  {
    ticks = 100;
    ReadPushButtons();  // every 10ms
  }
}

static void ReadPushButtons( void )
{
  static u08 uKeyState=0;
  static u08 ticks=0;
  static u08 repeat_ticks=0;
  u08 state;
  
  state = ~GEN_PUSH_BUTTON_PORT;
  state &= GEN_PUSH_BTN_MASK;
  if( !ROT_ENC_PUSH_BTN )
    state |= BTN_PUSH_ENC_BIT;
  else
    state &= ~BTN_PUSH_ENC_BIT;
  
  if( state != 0 && uKeyState == state )
  {
    if( ticks < 255 )
      ticks++;   // count ticks
    
    if ( ticks == BTN_LONG )
    {
      uButtons = uKeyState | BTN_LONG_MASK;
    }
  }
  else  // key releases or bouncing
  {
    if( ticks > BTN_SHORT && ticks < BTN_LONG)
    {
      uButtons = uKeyState;
    }
    repeat_ticks = 0;
    ticks = 0;
  }
  uKeyState = state;
}


/*
 * Einfacher ist jedoch eine Umwandlung des Gray-Code nach Binr, wie sie
 * in diesem 2.Programmbeispiel erfolgt:
 * 
 * Gray-Code   Binr-Code
 * 00          00 (0)     o.k.
 * 01          01 (1)     o.k.
 * 11          10 (2)     ndern
 * 10          11 (3)     ndern
 * 
 * Wie man sieht, mu man also nur das niederwertige Bit umdrehen, sobald
 * das hherwertige Bit gesetzt ist.
 * 
 * Diese Anweisung:
 *   if( PHASE_B )
 *     i ^= 3;
 * setzt das hherwertige Bit und invertiert das niederwertige, fertig ist
 * die Codeumwandlung.
 * 
 * Danach kann man nun die Differenz zwischen 2 aufeinanderfolgenden
 * Auslesungen bilden:
 *   i -= enc_last;
 */
// call cyclic to sample the rotary encoder signals
static void ReadEncoder( void )
{
  static s08 enc_last = 0x01;
  s08 i = 0;

  if( ROT_ENC_SIGA == 0 )
    i = 1;

  if( ROT_ENC_SIGB == 0 )
    i ^= 3;        // convert gray to binary

  i -= enc_last;      // difference new - last

  if( i & 1 ){        // bit 0 = value (1)
    enc_last += i;      // store new as next last

    enc_delta += (i & 2) - 1;    // bit 1 = direction (+/-)
  }
}
