/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdio.h>

#include "typedef.h"
#include "drvuart.h"

// Globale Variablen
static char TxBuff[TX_BUFFER_SIZE];
static char RxBuff[RX_BUFFER_SIZE];

static volatile uint8_t TxRead=0;		// Zeigt auf Zeichen, das gesendet werden soll
static uint8_t TxWrite=0;				// Zeigt auf nchste Schreibposition im Puffer

static uint8_t RxRead=0;            // Zeigt auf das nchste empf. Zeichen im Puffer
static volatile uint8_t RxWrite=0;  // Zeigt auf nchste Schreibposition im RX-Puffer

#ifdef __AVR_ATmega32__
#define EN_UDRE_IRQ()  (UCSRB |= (1<<UDRIE))    // Macro: UDRE Interrupt ein
#endif
#ifdef __AVR_ATmega644__
#define EN_UDRE_IRQ()  (UCSR0B |= (1<<UDRIE0))    // Macro: UDRE Interrupt ein
#endif

static FILE mystdout = FDEV_SETUP_STREAM(drvuart_UartPutC, NULL, _FDEV_SETUP_WRITE);
static FILE mystdin  = FDEV_SETUP_STREAM(NULL, drvuart_UartGetChar, _FDEV_SETUP_READ);

// Funktionen ffentlich

#ifdef __AVR_ATmega32__
void drvuart_InitModule(void) {
	UBRRL = (F_CPU/(16L*UART_BAUD_RATE))-1;
	UCSRB |= (1<<TXEN) | (1<<RXEN) | (1<<RXCIE);	// UART TX/RX und RX IRQ einschalten
	stdout = &mystdout;  // define our own stdio device
	stdin = &mystdin;
}

// TX-Interrupt Service Routine
ISR(USART_UDRE_vect) {
  if(TxRead != TxWrite) {
    uint8_t temp = (TxRead + 1) & TX_BUFFER_MASK;
    TxRead = temp;
    UDR = TxBuff[temp];                 // nchtes Zeichen aus Puffer senden
  } else UCSRB &= ~(1<<UDRIE);              // UDRE Interrupt aus
}

// RX-Interrupt Service Routine
ISR(USART_RXC_vect)
{
  u08 c = UDR;  // read character
  u08 temp = (RxWrite + 1) & RX_BUFFER_MASK;
  
  if( temp != RxRead )  // otherwise buffer overflow, trash the received character
  {
    RxBuff[RxWrite] = c;
    RxWrite = temp;
  }
}
#endif

#ifdef __AVR_ATmega644__
void drvuart_InitModule(void) {
  UBRR0L = (F_CPU/(16L*UART_BAUD_RATE))-1;
  UBRR0H = ((F_CPU/(16L*UART_BAUD_RATE))-1) >> 8;
  UCSR0B |= (1<<TXEN0) | (1<<RXEN0) | (1<<RXCIE0);  // UART TX/RX und RX IRQ einschalten
  /* Set frame format: 8data, 1stop bit, no parity */
  UCSR0C = (3<<UCSZ00);  
  stdout = &mystdout;  // define our own stdio device
  stdin = &mystdin;
}

// TX-Interrupt Service Routine
ISR(USART0_UDRE_vect) {
  if(TxRead != TxWrite) {
    uint8_t temp = (TxRead + 1) & TX_BUFFER_MASK;
    TxRead = temp;
    UDR0 = TxBuff[temp];                 // nchtes Zeichen aus Puffer senden
  } else UCSR0B &= ~(1<<UDRIE0);              // UDRE Interrupt aus
}

// RX-Interrupt Service Routine
ISR(USART0_RX_vect)
{
  u08 c = UDR0;  // read character
  u08 temp = (RxWrite + 1) & RX_BUFFER_MASK;
  
  if( temp != RxRead )  // otherwise buffer overflow, trash the received character
  {
    RxBuff[RxWrite] = c;
    RxWrite = temp;
  }
}
#endif
int drvuart_UartPutC(char data, FILE *stream)
{
	uint8_t temp = (TxWrite + 1) & TX_BUFFER_MASK;
	while(temp == TxRead);									// Warte bis Platz im Puffer
	TxBuff[temp] = data;
	TxWrite = temp;
	EN_UDRE_IRQ();									// UDRE Interrupt ein
	return 0;
}

void drvuart_UartPutS(const char *s) {
	while(*s) drvuart_UartPutC(*s++, NULL);
}

void drvuart_UartPutP(const char *progmem_s) {
	register char c;
	while((c = pgm_read_byte(progmem_s++))) drvuart_UartPutC(c, NULL);
}

int drvuart_UartGetChar( FILE *stream )
{
  u08 c;
  while(RxRead == RxWrite);
  uint8_t temp = (RxRead + 1) & RX_BUFFER_MASK;
  c = RxBuff[RxRead];  // nchtes Zeichen aus Puffer lesen
  RxRead = temp;
  return c;
}

u08 kbhit( void )
{
  return (RxRead != RxWrite);
}

