/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* test fr winavr 20080512
-fno-tree-scev-cprop
-fno-inline-small-functions
-mcall-prologues
*/

#include <avr/pgmspace.h>
#include <string.h>
#include <stdio.h>

#include "typedef.h"
#include "sysdef.h"
#include "drvlcd.h"
#include "hwdef.h"
#include "iotimer.h"
#include "drvtimer.h"
#include "drvheat.h"
#include "drvuart.h"
#include "ioadc.h"
#include "pid.h"
#include "drveeprom.h"
#include "drvtemp.h"
#include "regulator.h"
//#include "parser_lib.h"
#include "cmdline.h"
#include "drvkeys.h"
#include "display.h"
#include "xitoa.h"

#define VAL_KP 1
#define VAL_KI 2
#define VAL_KD 3
#define VAL_TEMP 4

static char PROGMEM cCRLF[] = "\r\n";
static volatile u08 uTimerTick = 0;

static void TimerTick( void );
static void DumpValues( void );
static void InitAll( void );
void second_loop( void );
void helpFunction(void);
void dumpArgsStr(void);
void dumpArgsInt(void);
void dumpArgsHex(void);
void Parm_SetKp( void );
void Parm_SetKi( void );
void Parm_SetKd( void );
void Parm_SetTemp( void );
static void SetKValues( u08 uPar );
static void EnableTempDump( void );
static void SetHeatPwrAbs( void );
static void KeyInterpreter( void );

static void InitAll( void )
{
  lcd_init();
  Iotimer_InitModule();
  drvtim_InitModule();
  drvuart_InitModule();
  Iotimer_RegisterTimer0Fct( TimerTick );
  drveeprom_InitModule();
  drvkeys_InitModule();
  display_InitModule();
  drvheat_InitModule();
  ioadc_InitModule();
  drvtemp_InitModule();
  reg_InitModule();
  PID_InitModule();
  //vParserInit ();
  
  // initialize cmdline system
  cmdlineInit();

  // direct cmdline output to uart (serial port)
  cmdlineSetOutputFunc( (void (*)(unsigned char ))  drvuart_UartPutC);

  // add commands to the command database
  cmdlineAddCommand("help",   helpFunction);
  cmdlineAddCommand("setkp", Parm_SetKp);
  cmdlineAddCommand("setki", Parm_SetKi);
  cmdlineAddCommand("setkd", Parm_SetKd);
  cmdlineAddCommand("sett", Parm_SetTemp);
  cmdlineAddCommand("dtemp", EnableTempDump);
  cmdlineAddCommand("dpar", DumpValues);
  cmdlineAddCommand("setpwr", SetHeatPwrAbs);
  

  // send a CR to cmdline input to stimulate a prompt
  cmdlineInputFunc('\r');
  
#ifdef DEBUG
  // disable test port pin (set to 0)
  cbi(TEST_PORT, TEST_BIT_NO);
  // initialize test port pin to output
  sbi(TEST_PORT_DDR, TEST_BIT_NO);
  
#endif
  
  sei();
}

int main( void )
{
  xfunc_out = (void (*)(char))drvuart_UartPutC;
  InitAll();
  
  drvuart_UartPutP(PSTR("Loetstation V0.1.0\r\n"));
  drvuart_UartPutP(PSTR("Init done, ready\r\n"));
  
  drvtim_TimerAdd( second_loop, 1000 );

  drvheat_SetHeatingPower( E_SGUN1, 0 );
  drvheat_SetHeatingPower( E_SGUN2, 0 );
  
  while ( 1 )
  {
#ifdef xDEBUG    
    if( TEST_BIT )
      TEST_BIT = 0;
    else
      TEST_BIT = 1;
#endif    
    
    if( uTimerTick )
    {
      uTimerTick = 0;
      drvtim_TimerTick();
    }  
    drveeprom_Update();
    drvheat_Task();
    reg_Task();
    display_Task();
    KeyInterpreter();

    // pass characters received on the uart (serial port)
    // into the cmdline processor
    if( kbhit() )
      cmdlineInputFunc(drvuart_UartGetChar(0));

    // run the cmdline execution functions
    cmdlineMainLoop();
    
   // if( kbhit() )
      //vParserProcessChar ( (UCHAR)drvuart_UartGetChar(0) );
  }
  return 0;
}

void second_loop( void )
{
  drvtim_TimerAdd( second_loop, 1 );
  //printf("Second\r\n");
#ifdef DEBUG    
    if( TEST_BIT )
      TEST_BIT = 0;
    else
      TEST_BIT = 1;
#endif    
  
}


static void TimerTick( void )
{
  #define TICK_DIVIDER  10
  static u08 ticks = TICK_DIVIDER;
  
  uTimerTick = 1;
  if( (--ticks) == 0)
  {
    //drvkeys_task();
    ticks = TICK_DIVIDER;
  }
}

void helpFunction(void)
{
  drvuart_UartPutP(PSTR("setk?  : (pid) gun value\r\n"));
  drvuart_UartPutP(PSTR("sett   : gun temperature\r\n"));
  drvuart_UartPutP(PSTR("setpwr : gun absolute power\r\n"));
  drvuart_UartPutP(PSTR("dtemp  : gun 0/1\r\n"));
  drvuart_UartPutP(PSTR("dpar   : dump eeprom\r\n"));
}

void Parm_SetKp( void )
{
  // SETKI
  SetKValues( VAL_KP );
}

void Parm_SetKi( void )
{
  // SETKI
  SetKValues( VAL_KI );
}

void Parm_SetKd( void )
{
  // SETKD nr value
  SetKValues( VAL_KD );
}

void Parm_SetTemp( void )
{
  // SETKD nr value
  SetKValues( VAL_TEMP );
}

static void SetKValues( u08 uPar )
{
  //double fVal;
  u16 uVal;
  u08 sgun;
  
  sgun = cmdlineGetArgInt(1);
  //fVal  = cmdlineGetArgFloat(2);
  uVal  = cmdlineGetArgInt(2);

  if( sgun == 1 || sgun == 2 )
  {
    switch( uPar )
    {
    case VAL_KP:
      drveeprom_GetParameter()->tEeprom->fKp[sgun-1] = uVal;
      break;
    case VAL_KI:
      drveeprom_GetParameter()->tEeprom->fKi[sgun-1] = uVal;
      break;
    case VAL_KD:
      drveeprom_GetParameter()->tEeprom->fKd[sgun-1] = uVal;
      break;
    case VAL_TEMP:
      drveeprom_GetParameter()->tEeprom->uNominalTemp[sgun-1] = uVal;
    }
    drveeprom_Write();
    PID_RefreshKValues();
    DumpValues();
  }
  else
    drvuart_UartPutP(PSTR("Error: invalid values\r\n"));
}

static void DumpValues( void )
{
  T_PARAM_ALL *sAllData = drveeprom_GetParameter();
  T_EEPROM *sEepromData = sAllData->tEeprom;
  xputs(PSTR("\r\nDump Eeprom data:\r\n"));
  xprintf(PSTR("ActGun : %3d\r\n"),sAllData->uActGun);
  xprintf(PSTR("GunPwr : %s, %s\r\n"), sEepromData->uGunOff[0] ? "OFF" : "ON", sEepromData->uGunOff[1] ? "OFF" : "ON" );
  xprintf(PSTR("NomScr : %3d\r\n"), sEepromData->uNomScreen);
  xprintf(PSTR("CalLow : %3d, %3d\r\n"), sEepromData->uCalLow[0], sEepromData->uCalLow[1]);
  xprintf(PSTR("CalLow : %3d, %3d\r\n"), sEepromData->uCalHigh[0], sEepromData->uCalHigh[1]);
  xprintf(PSTR("NomTemp: %3d, %3d\r\n"),  sEepromData->uNominalTemp[0], sEepromData->uNominalTemp[1]);
  xprintf(PSTR("PID Kp : %3d, %3d\r\n"),  sEepromData->fKp[0], sEepromData->fKp[1]); 
  xprintf(PSTR("PID Ki : %3d, %3d\r\n"),  sEepromData->fKi[0], sEepromData->fKi[1]); 
  xprintf(PSTR("PID Kd : %3d, %3d\r\n"),  sEepromData->fKd[0], sEepromData->fKd[1]); 
}

static void EnableTempDump( void )
{
  extern u08 reg_dump_temp[];
  u08 uVal;
  u08 sgun;
  
  sgun = cmdlineGetArgInt(1);
  uVal  = cmdlineGetArgInt(2);

  if( sgun == 1 || sgun == 2 )
  {
    reg_dump_temp[sgun-1] = uVal;
  }
  else
    drvuart_UartPutP(PSTR("Error: invalid values\r\n"));
}

static void SetHeatPwrAbs( void )
{
  extern u08 heat_pwr_abs[];
  u08 uVal;
  u08 sgun;
  
  sgun = cmdlineGetArgInt(1);
  uVal  = cmdlineGetArgInt(2);

  if( sgun == 1 || sgun == 2 )
  {
    heat_pwr_abs[sgun-1] = uVal;
  }
  else
    drvuart_UartPutP(PSTR("Error: invalid values\r\n"));
}

static void KeyInterpreter( void )
{
  char uKey;
  T_PARAM_ALL *sParamAll;
  
  if( drvkeys_kbhit() )
  {
    sParamAll = drveeprom_GetParameter();
    uKey = drvkeys_getchar();
    
    if( !display_ErrorActive() )
    {
      switch( uKey )
      {
      case '<':
        if( sParamAll->tEeprom->uNominalTemp[sParamAll->uActGun] > 150 )
          sParamAll->tEeprom->uNominalTemp[sParamAll->uActGun] -= 5;
        drveeprom_Write();
        break;
      case '>':
        if( sParamAll->tEeprom->uNominalTemp[sParamAll->uActGun] < 450 )
          sParamAll->tEeprom->uNominalTemp[sParamAll->uActGun] += 5;
        drveeprom_Write();
        break;
      case 'e':  
        sParamAll->uActGun ^= 1;
        break;
      case 'E':  
        sParamAll->tEeprom->uGunOff[sParamAll->uActGun] ^= TRUE;
        drveeprom_Write();
      }
    }
    else
      display_ResetErrorText();
  }
}
